PROGRAM distran
!********************************************************************** 
!       To determine the flows and stratification in a "filling box" pro
!       FOR A VERTICALLY DISTRIBUTED SOURCE                             
!       virtual source with finite  vol flux                 
!       - all variables are dimensioal.                                  
!		initially written by Paul Cooper, University of Wollongong
!		revised and added functionality by Ole Joergen Myrtroeen, NTNU
!********************************************************************** 
!        All vertical fluxes are per unit width of the source.
USE shared_var 
IMPLICIT NONE
                                                               
  !Initialize variables (set them to zero)
  CALL init_var()
  !Open the relevant files
  CALL openfiles()                                      
  !Give the variables their respective values:
  CALL start_values()
  ! UI for starting the program:
  !CALL menu_choice()

!************************************************************           
!     Now set current value of A* (this is the main loop)               
!************************************************************
DO ! mainloop
            
    iAst = iAst+1
    IF (iAst > iAstmax) EXIT
    Astar = Ast(iAst)
	Ast_non_dim = Astar/(alpha*xL*H)  ! non-dimensional version of A*
    iPrcount=1 !iPrcount is the print counter for this value of Astar
    
    CALL writefiles() ! write initial info to files
                                 
    !First front arrives at top of box - calc                          
    !volume flow rate and g' at top of box:
    
    i=1   ! position counter
    j=1   ! time counter
	time=0.
    jx=1  ! tot. no. of layers
    xQ(1)=xQ0 ! Volume flux
    xM(1)=xM0 ! Momentum flux (all per unit horiz length of source)
    xF(1)=xF0 ! Buoyancy flux

    
    !******************************************
    !IMPORTANT: Inserting stratified ambient subroutine here!
    !******************************************
    
    IF (stratified_on == 1) THEN
        CALL ambient_stratification
    END IF
    
	!miss out the next section as this was originally used when we
	! started with no. layers in the box at time = 0

	GO TO 1000

    !******************************************                                                           
    !Plume vol. & mom. at top of box                                   
    !******************************************

         pd_layers = jx
	CALL pressure_distribution()

     IF (varying_buoyancy == 1) THEN
         ! F_s is buoyancy flux for current position (numerical interface)
         xFprime = F_s(i)
      ELSE
        ! (For plotting reasons:)
         F_s(i) = xFprime
      END IF
 
    CALL euler (z0, xQ(1), xM(1), H, xQ(2), xM(2), alpha, xF(1), xFprime)   
    
	IF ((varying_buoyancy == 1) .AND. (stratified_on == 1))  THEN
		!gprimea(jx-1) = gprimea(jx-1) + xFprime*H/xQ(2) !find gprime of the current top layer
        !gprimea(jx-1) = gprimea(jx-2) + xFprime*H/xQ(2)
    ELSE IF ((varying_buoyancy == 0) .AND. (stratified_on == 1)) THEN
        !gprimea(jx) = gprimea(jx) + xFprime*H/xQ(2)
    ELSE IF ((varying_buoyancy == 1) .AND. (stratified_on == 0)) THEN
        gprimea(2) = gprimea(1) + xFprime/xQ(2)
    ELSE IF ((varying_buoyancy == 0) .AND. (stratified_on == 0)) THEN
        gprimea(2) = gprimea(1) + xFprime*H/xQ(2) !find gprime of the current top layer
    END IF
    
    xQvent(1)=0.0 !Find how much fluid has left top of box
  
    !Find depth of newly formed top layer:
    
    IF (stratified_on == 1) THEN
      
      U(jx-1) = (xQvent(1)-xQ(2))*xL/S  !velocity of first front 
      deltat = dzetalim*H/ABS(U(jx-1))  !time step interval [s]
      !z(jx-1) = H + U(jx-1)*deltat
      !U(jx)=(xQvent(1)-xQ(1))*xL/S !U(1) is velocity of initial fluid in box past source              
                                !     out through lower opening (if any)      
    ELSE
      
      U(2) = (xQvent(1)-xQ(2))*xL/S  !velocity of first front 
      deltat = dzetalim*H/ABS(U(2))  !time step interval [s]
      z(2) = H + U(2)*deltat           !new position of first front after deltat secs
      z(j+2) = H
      U(1)=(xQvent(1)-xQ(1))*xL/S !U(1) is velocity of initial fluid in box past source              
                                !     out through lower opening (if any)       
    END IF
                                 
  
    IF (stratified_on == 1) THEN
      !z(jx) = U(jx)*deltat  !z(jx) + U(jx)*deltat     
    ELSE                      
      z(1)=U(1)*deltat !new position of bottom of this interface after deltat secs
    END IF
    
    xfirst(1) = z(2) !xfirst keeps a record of the first front position 
    time = deltat    !
    xtime(1) = time
           
    j =2 ! time counter, (j=1 already calc)
    IF (stratified_on == 0) THEN
      jx=2 ! current no. of numerical interfaces in the box
    ELSE
      jx = jx + 1
    END IF
    !**********************************************************************************************         
    !START OF TIME LOOP
    !  determine plume development through                           
    !  multiple layers in box jth timestep        
    !**********************************************************************************************                         
    
 1000	CONTINUE
 	
	j = 1  ! time counter

	timeLoop: DO
        
		IF (iPrcount > iPrmax) EXIT
        
        jHcount=0 
        jCross=0                                                              
        ndtran=0 !the number of points at which detrainment occurs
		ldtran = 0
        zdtran = 0

        CALL check_thickness ! has jx here. s.r. that checks if any layer is too thin.				
             			     ! if so, the s.r. removes that layer. and jx = jx - 1
                                                                             
        i=1 ! position counter set to one
        
        !Flow through original ambient layer:
        xQ(1)=xQ0
        xM(1)=xM0
        xF(1)=xF0
        z(1)=z0

         pd_layers = jx
        
		IF (varying_buoyancy == 1) THEN
			CALL pressure_distribution()
			! Calc the pressure difference and fluxes across membrane as function of height
		END IF

		CALL i_loop() ! step through each layer calculating vol. flow, xF etc
		! i-loop checks if there has been de-entrainment and finds the position
        
        ! Calc gprime of new top layer and the buoyancy driven flow through vent.
		CALL gprimea_calc()
        ! Note: if plume is de-entraining at top treat carefully!

        ! First find velocity of fastest moving interface:
		CALL umax_find()
        ! returns (scalar) Umax as the max velocity
        
        !Now calculate required timestep                                                                     
        deltat = dzetalim*H/Umax
        !write(10,*)deltat                                              
        time = time + deltat 
        xtime(j) = time
        
		WRITE(*,*)'Next time',time,' j=',j,' jx=',jx                                       
                                                                           
        !FIND NEW LAYER HEIGHTS z(i) Calculate new interface positions
        DO jj=1, jx+1    
          z(jj)=z(jj) + U(jj)*deltat
        END DO
 
 GO TO 2122     
 			WRITE(*,*) ' new layer heights, jx=',jx,' i=',i
			WRITE(*,*) ' n_e,  z(n_e),       gprimea(n_e),     xQ(n_e),     xF(n_e),     F_s(n_e)'
 			DO n_e = 1, jx+1
			WRITE(*,247) n_e,z(n_e),gprimea(n_e),xQ(n_e),xF(n_e),F_s(n_e)
			END DO
 2122	CONTINUE

        xfirst(j)=z(2)
            
        !Check if all interfaces are still within the box; if not, remove them. AND
		! Increase no. of interfaces for next loop jx = jx +1
        CALL interface_check()
      
        !Write out current stratification                                  
        IF ((time-Prtime(iPrcount)) >= &
              &(0.001*Prtime(iPrcount))) THEN
 
            !Set up print-out matrices                                         
            CALL matrices()
      
            ! Write the results to 'all.res' :
            CALL write_results()

            !Step to next value of Prtime before printing again
            IF (iPrcount >= iPrmax) EXIT ! quit if all required values of time are printed 
            iPrcount = iPrcount + 1
            
        END IF                                                                      
 
 GO TO 2222
		IF (j >= 190) THEN	
 			WRITE(*,*) ' '
			WRITE(*,*) ' '
			WRITE(*,*)' end of the i_loop, j=',j,' jx=',jx,' i=',i
			PAUSE
			DO n_e = 1, jx
			WRITE(*,247) n_e,z(n_e),gprimea(n_e),xQ(n_e),xF(n_e),F_s(n_e)
			END DO
			WRITE(*,*) ' n_e,  z(n_e),       gprimea(n_e),     xQ(n_e),     xF(n_e),     F_s(n_e)'
 			  247 FORMAT (1x,I3,1x,e13.8,1x,4(1x,e11.4))
			Pause
		END IF

 2222 CONTINUE                                 
        j = j + 1 !Increase timestep, and start from top                                     


    END DO timeloop
    !**********************************************************************************************  
    !**********************************************************************************************  
 

 
                                                
    IF (iAst < iAstmax) CYCLE

WRITE(7,*)
WRITE(7,*) ' n_e,z(n_e),z(n_e+1),gprimea(n_e),g_e(n_e),Q_s(n_e),F_s(n_e)'
DO n_e = 1, (pd_layers)
WRITE(7,245) n_e,z(n_e),z(n_e+1),gprimea(n_e),g_e(n_e),Q_s(n_e),F_s(n_e)
END DO
245 FORMAT (1x,I3,1x,6(1x,e10.3))


END DO ! mainloop


END PROGRAM distran

SUBROUTINE gaussian_area(z0, z, Area, sigma, deentrain, value)
IMPLICIT NONE
! Integrates a Gaussian curve of area = 1, to find the area
! under it. O-J Myrtroeen 2007-03-19.

! Calculates the area y under a Gaussian curve,
! from xstart to xstop
REAL, INTENT(IN) :: z0, z, sigma, deentrain
REAL, INTENT(OUT) :: Area,  value
REAL :: pi, step, n, y, x
PARAMETER (pi = 3.141592653589790)

! Properties of the Gaussian curve:
!mju = 0   ! average, actually the height of neutral buoyancy
y = 0     ! the y value at a point x, f(x, sigma)
Area = 0  ! the area

n = 100 ! number of iterations
step = ABS(z-z0) / n ! step size

! Numerical integration of the Gaussian curve (with area = 1):
IF (z0 == z) THEN
  Area = 0.
  value = 0.
  RETURN
END IF
  
DO x = z0, (z-step), step
  
    y = y + ( 1/(sigma * SQRT(2.0*pi)) ) * EXP(-(0.5)*(((x-deentrain)/sigma)**2))
    
END DO

    !value = ( 1/(sigma * SQRT(2.0*pi)) ) * EXP(-(0.5)*(((z-mju)/sigma)**2))
	value = EXP(-(0.5)*(((z-deentrain)/sigma)**2))

Area = y * step ! The total area between xstart and xstop

END SUBROUTINE gaussian_area