	PROGRAM distran
!********************************************************************** 
!       To determine the flows and stratification in a "filling box" pro
!       FOR A VERTICALLY DISTRIBUTED SOURCE                             
!       virtual source with zero  vol and momentum flux                 
!       - all variables are primitive.                                  
!		initially written by Paul Cooper, University of Wollongong
!		revised and added functionality by Ole Joergen Myrtroeen, NTNU
!********************************************************************** 
	USE shared_var 
	IMPLICIT NONE
                                                               
  !Initialize variables (set them to zero)
	CALL init_var()
  !Open the relevant files
	CALL openfiles()                                      
  !Give the variables their respective values:
	CALL start_values()
  ! UI for starting the program:
	CALL menu_choice()

!************************************************************           
!     Now set current value of A* (this is the main loop)               
!************************************************************
	DO  !mainloop: 
            
	iAst = iAst+1
	IF (iAst > iAstmax) EXIT
	Astar = Ast(iAst)
	AstarH = Astar/H**2. 
	starHL = Astar/(H*xL) 
	iPrcount=1 !iPrcount is the print counter for this value of Astar
    
	CALL writefiles() ! write initial info to files
                                 
    !First front arrives at top of box - calc                          
    !volume flow rate and g' at top of box:
    
	i=1   ! position counter
	j=1   ! time counter
	jx=1  ! tot. no. of layers
	xQ(1)=xQ0 ! Volume flux
	xM(1)=xM0 ! Momentum flux (all per unit horiz length of source)
	xF(1)=xF0 ! Buoyancy flux
    
    !******************************************
    !IMPORTANT: Inserting stratified ambient subroutine here!
    !******************************************
!$$$$$$     IF (stratified_on == 1) THEN
!$$$$$$         CALL ambient_stratification
!$$$$$$     END IF
    
    !******************************************                                                           
    !Plume vol. & mom. at top of box                                   
    !******************************************                                            
	CALL euler (z0, xQ(1), xM(1), H, xQ(2), xM(2), alpha, xF(1), xFprime)   
    
	!IF (stratified_on == 1) THEN
	!	gprimea(2) = gprimea(2) + xFprime*H/xQ(2) !find gprime of the current top layer
    !ELSE
    	gprimea(2) = gprimea(1) + xFprime*H/xQ(2) !find gprime of the current top layer
    !END IF
    
	xQvent(1)=0.0 !Find how much fluid has left top of box
  
    !Find depth of newly formed top layer:
	U(2) = (xQvent(1)-xQ(2))*xL/S  !velocity of first front 
	deltat = dzetalim*H/ABS(U(2))  !time step interval [s]
	z(2) = H + U(2)*deltat           !new position of first front after deltat secs
	z(j+2) = H                     
                                 
    !Find how much fluid has left bottom of box:                        
	U(1)=(xQvent(1)-xQ(1))*xL/S !U(1) is velocity of initial fluid in box past source              
                                !     out through lower opening (if any)        
	z(1)=U(1)*deltat !new position of bottom of this interface after deltat secs
  
	xfirst(1) = z(2) !xfirst keeps a record of the first front position 
	time = deltat    !
	xtime(1) = time
    
	WRITE(8,*) ''
	WRITE(8,*) ' j ', 'jx ', 'time ', 'z(ii) ', 'U(ii) '
	WRITE(8,*) 
	WRITE(8,360) j, jx, time
360	FORMAT(1X, I4, 1X, I4, 1X, e10.4)
	WRITE(8,*) 'z(ii) ', 'U(ii) '
       
    
	j =2 ! time counter, (j=1 already calc)
	jx=2 ! current no. of numerical interfaces in the box
    
    !**********************************************************************************************         
    !START OF TIME LOOP
    !  determine plume development through                           
    !  multiple layers in box jth timestep        
    !**********************************************************************************************                         
	timeLoop: DO
        
		IF (iPrcount > iPrmax) EXIT
        
        jHcount=0 
        jCross=0                                                              
        ndtran=0 !the number of points at which detrainment occurs
                                                                           
    	!Clear the position of de-entrainment vector
		ldtran = 0
        zdtran = 0

        CALL check_thickness ! has jx here. s.r. that checks if any layer is too thin.				
             			     ! if so, the s.r. removes that layer. and jx = jx - 1
                                                                             
        i=1 ! position counter set to one
        
        !Flow through original ambient layer:
        xQ(1)=xQ0
        xM(1)=xM0
        xF(1)=xF0
        z(1)=z0


        CALL i_loop() ! has jx here
		! i-loop checks if there has been de-entrainment and finds the position
        
        ! Calc gprime of new top layer and the buoyancy driven flow through vent.
		CALL gprimea_calc()
        ! Note: if plume is de-entraining at top treat carefully!

        ! First find velocity of fastest moving interface:
		CALL umax_find()
        ! returns (scalar) Umax as the max velocity
        
        !Now calculate required timestep                                                                     
        deltat = dzetalim*H/Umax
        !write(10,*)deltat                                              
        time = time + deltat 
        xtime(j) = time
        
        WRITE(*,*)' Astar=',Astar,' time=',time,' j=',j,' jx=',jx                                       
                                                                           
        !FIND NEW LAYER HEIGHTS z(i) Calculate new interface positions
        DO jj=1, jx+1    
          z(jj)=z(jj) + U(jj)*deltat
        END DO
      
        xfirst(j)=z(2)
            
        !Check if all interfaces are still within the box; if not, remove them.
        CALL interface_check()
      
        !Write out current stratification                                  
        IF ((time-Prtime(iPrcount)) >= (0.001*Prtime(iPrcount))) THEN
              
              !Print warnings about interface positions. 
              IF (jHcount /= 0) THEN
                 WRITE(10, 3334) jHcount, jj
 3334				FORMAT (1X, 'Warning: the new position of ', I4, 'interface were > H for jj = ', I4)	
              END IF

              IF (jCross /= 0) THEN
                 WRITE(10, 3333) jCross, jj
3333 		FORMAT(1X, 'Warning: the new positions of',	I4, ' interfaces are now lower than the neighbour'' for jj =', I4)	
              END IF
        
            !Set up print-out matrices                                         
            CALL matrices()
      
            ! Write the results to 'all.res' :
            CALL write_results()

            !Step to next value of Prtime before printing again
            IF (iPrcount >= iPrmax) EXIT ! quit if all required values of time are printed 
            iPrcount = iPrcount + 1
            
        END IF                                                                      
                                   
        j = j + 1 !Increase timestep, and start from top                                     
        
	END DO timeloop
    !**********************************************************************************************  
    !**********************************************************************************************  

    !Print out final stratification results
	WRITE (10,*)''                                           
	WRITE (10,*)'time, xQvent(j), xF, xfirst'
        
	DO iii=1,j      
        WRITE(10,1210)xtime(iii),xQvent(iii),xF(iii),xfirst(iii)   
1210	FORMAT(1x,4(1x,e10.4))
	END DO

	DO iii = 1, jx
        WRITE(9,1220)z(iii),xF(iii)
1220	FORMAT(1x,2(1x,e10.4))
	END DO

                                                 
	IF (iAst < iAstmax) CYCLE


	END DO !mainloop: 


	END PROGRAM distran

!$$$$$$                                        
! Here follows the subroutine euler
!$$$$$$ 
                                                                                                                   
	SUBROUTINE euler(z0,xQ0, xM0,z ,xQ,xM, alpha, xF, xFprime)
  !*******************************************************                
  !        Simple Euler method to solve DE's for                          
  !        plume volume flow, xq, and momentum,                           
  !        xm (xf = buoyancy flux).                                                      
  !*******************************************************
	IMPLICIT NONE
	REAL, INTENT(IN) :: z0, xQ0, xM0, z, xF, alpha, xFprime
	REAL, INTENT(OUT) :: xQ, xM 
  !internal variables:
	REAL :: h, xn, xkt, zn
	REAL :: xQn, xFn, xMn, xQnew, xMnew
	INTEGER :: ii
  
  !xn = no. of increments in x 
  !xn=1000.
	xn = 50.
  
!$$$$$$   IF ((z-z0) < 1.e-7) THEN
!$$$$$$       xn=100. 
!$$$$$$   ELSEIF ((z-z0) < 1.e-6) THEN
!$$$$$$       xn=200. 
!$$$$$$   END IF
  
  ! Setting the local variables
	zn = z0     !height of layer
	xQn = xQ0   !volume flux into layer
	xMn = xM0   !momentum flux into layer
	xkt = 4. !4 ! non-linear mesh exponent (distribution of steps)
  !h=(z-z0)/xn !distance per step (linear version)
  
  !LOOP INCREMENTING Z
 
	DO ii = 1, xn
      ! Setting the next z
      zn = z0 + (z-z0)*(ii/xn)**xkt
      
      ! calc the distance of this step
      h = zn - (z0 + (z-z0)*((ii-1.)/xn)**xkt)
                                       
      IF (xMn < (1.e-12)) THEN
          xMn=1.e-12 
      END IF
      IF (xQn < (1.e-12)) THEN
          xQn=1.e-12 
      END IF
                                        
      xFn = xF + zn*xFprime
      xQnew = xQn + alpha*xMn/xQn*h
       
      xMnew = xMn + xQn*xFn/xMn*h 
                                
      !gprime=(xFn+h*xFprime)/xQn                                      
      zn = zn + h
      xQn = xQnew 
      xMn = xMnew

	END DO
  
	xQ=xQn !update volume flux
	xM=xMn !update momentum flux
  
	END SUBROUTINE euler

	SUBROUTINE gaussian_area(z0, z, Area, sigma, deentrain, value)
	IMPLICIT NONE
! Integrates a Gaussian curve of area = 1, to find the area
! under it. O-J Myrtroeen 2007-03-19.

! Calculates the area y under a Gaussian curve,
! from xstart to xstop
	REAL, INTENT(IN) :: z0, z, sigma, deentrain
	REAL, INTENT(OUT) :: Area,  value
	REAL :: pi, step, n, y, x
	PARAMETER (pi = 3.141592653589790)

! Properties of the Gaussian curve:
!mju = 0   ! average, actually the height of neutral buoyancy
	y = 0     ! the y value at a point x, f(x, sigma)
	Area = 0  ! the area

	n = 1000 ! number of iterations
	step = ABS(z-z0) / n ! step size

! Numerical integration of the Gaussian curve (with area = 1):
	IF (z0 == z) THEN
	Area = 0.
	value = 0.
		RETURN
	END IF
  
	DO x = z0, (z-step), step
  
	y = y + ( 1/(sigma * SQRT(2.0*pi)) ) * EXP(-(0.5)*(((x-deentrain)/sigma)**2))
    
	END DO

    !value = ( 1/(sigma * SQRT(2.0*pi)) ) * EXP(-(0.5)*(((z-mju)/sigma)**2))
	value = EXP(-(0.5)*(((z-deentrain)/sigma)**2))

	Area = y * step ! The total area between xstart and xstop

	END SUBROUTINE gaussian_area